/**
 * Copyright 2021 Huawei Technologies Co., Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef GE_GRAPH_PASSES_INFER_BASE_PASS_H_
#define GE_GRAPH_PASSES_INFER_BASE_PASS_H_

#include "graph/passes/base_pass.h"

namespace ge {
class InferBasePass : public BaseNodePass {
 public:
  Status Run(NodePtr &node) override;
  graphStatus InferAndUpdate(NodePtr &node, bool before_subgraph, std::set<NodePtr> &changed_nodes);
  void PrintInOutTensorShape(const NodePtr &node, const std::string &phase);

 protected:
  virtual graphStatus Infer(NodePtr &node) = 0;
  virtual bool TensorDescChanged(const GeTensorDescPtr &src, const GeTensorDescPtr &dst) = 0;
  virtual graphStatus UpdateInputDescAttr(const GeTensorDescPtr &src, GeTensorDescPtr &dst, bool &changed);
  virtual bool NeedInfer(const NodePtr &node);
  virtual void AnalyzeFailedInfo(const NodePtr &node);
  virtual Status DoRepassForLoopNode(NodePtr &node);   // only for infershape, will be deleted
  virtual graphStatus UpdatePeerInputs(NodePtr &node); // only for infershape, will be deleted

 private:
  void AddChangedNodesImmediateRepass(std::set<NodePtr> &changed_nodes);
  graphStatus UpdateCurOpInputDesc(const NodePtr &node_ptr);
  bool ContainsSubgraph(const NodePtr &node);
  std::vector<ComputeGraphPtr> GetCurNodeSubgraphs(const NodePtr &node);
  graphStatus UpdateTensorDescToSubgraphData(NodePtr &node, std::set<NodePtr> &changed_nodes);
  graphStatus UpdateTensorDescToParentNode(NodePtr &node, std::set<NodePtr> &changed_nodes);
  graphStatus UpdateParentNodeForWhile(NodePtr &node, std::vector<std::vector<GeTensorDesc>> &ref_data_tensors,
                                       std::vector<std::vector<GeTensorDesc>> &ref_out_tensors,
                                       std::set<NodePtr> &changed_nodes);
  graphStatus UpdateParentNodeForBranch(NodePtr &node, std::vector<std::vector<GeTensorDesc>> &ref_out_tensors,
                                        std::set<NodePtr> &changed_nodes);
  graphStatus UpdateOutputForMultiBatch(NodePtr &node, std::vector<std::vector<GeTensorDesc>> &ref_out_tensors,
                                        std::set<NodePtr> &changed_nodes);
};
}  // namespace ge
#endif  // GE_GRAPH_PASSES_INFER_BASE_PASS_H_
